/***********************************************************************
 version 1 - 15/01/98 

 TriangleTree function

 Alain Guenoche (1), Laurent Duret (2), Bruno Leclerc (3)
 The triangles method to build phylogenetic trees from incomplete
 distance matrices.

 (1) Laboratoire d'Informatique de Marseille, 
     163 Av de Luminy, 13009 Marseille, France
     guenoche@lim.univ-mrs.fr

 (2) Laboratoire BGBP - UMR CNRS 5558     
     Universite Claude Bernard - Lyon 1   
     43 Bd du 11 Novembre 1918    
     69622 Villeurbanne Cedex, France           
     duret@biomserv.univ-lyon1.fr

 (3) Centre d'Analyse et de Mathematique Sociales
     Ecole des Hautes Etudes en Sciences Sociales
     54 Bd Raspail
     75270 Paris cedex 06, France
     leclerc@ehess.fr

 cc -o TriangleTree  TriangleTree.c -lm

************************************************************************/

#include <stdio.h>
#include <stdlib.h>   
#include <string.h>
#include <math.h>


#define MAXLNAME 30
#define EPSILON   0.000001
#define MEPSILON -0.000001


/***************** global variables **********************************/
struct TABDIST {
	int inc;		/* number of unknown distances  */
	int pl;			/* order in the tree            */
	int idx;		/* index                        */
	int bestar;		/* closest edge                 */
	double bestd;		/* distance to the closest edge */
	} *Xt;

struct TREE {
	int Pred;		
	int Suc;		
	int Fre;		
	double Long;		
	int Mark;		
	int Sd;			
	int Sf;			
	int Ch;			
	} *tree;


double **Xdst;			/* distances table */
char **Xmn;			/* sequence names  */
int NbXt;			/* number of sequences in X*/
int NbYt;			/* number of sequences in Y*/
int Nbc; 			/* number of seq. with complete distances */
double Dmax;			/* max distance */
int Ns;				
int Nsmax;			
int Na;				
int NbPl;			
int Nb0;			
int Nbm;			

/******************* functions *****************/

int  loadXt(double **dist, char **nom, int nbdist);
void sortXt(void);
int  SelectTriple(int *x, int *y, int *z);
int  Growtree(int s, int t, int u);
void UpDist(int cand);
void Path(int t, int u);
void SwapDist(int ii, int jj);
void CompTree(void);
void FormatTree(char *string);
int  TriangleTree(double ** dist, char** nom, int nbelt, char* treepar);
void FreeTab(void);
char *check_alloc(int nbrelt, int sizelt);

/****************************************************************/


/*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                           ;
;                         TriangleTree function                             ;
;                                                                           ;
;                         "dist" is the distance matrix                     ;
;                         "nom" is the list of sequence names               ;
;                         "nbelt" is the number of sequences                ;
;                         "treepar" is the phylogenetic tree in  NEWSWICK   ;
;                         (parenthesis) format.                             ;
;                                                                           ;
;                         TriangleTree returns 1 if normal end, 0 otherwise ;
;                                                                           ;
;                         Unknown distances are allowed in the distance     ;
;                         matrix. They should be indicated  by negative     ;
;                         values.                                           ;
;                                                                           ;
\*;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;*/


int TriangleTree(double ** dist, char** nom, int nbelt, char* treepar)

{
int x, y, z;
int kk, ii, jj, aa, ar, Sd, Sf, cand, som, somi, somj;
double lkk, lmax;
float frq;

treepar[0] = 0;

/* load data */
if(loadXt(dist, nom, nbelt) == 0) {
	FreeTab();
	return(0);
	}

/* sort data */
sortXt();


/* choose 3 elements in Y */

if(SelectTriple(&x, &y, &z) == 0) {
	FreeTab();
	return(0);
	}

/* initialisation of the 2-trees */
tree[1].Sd=x;
tree[1].Sf=y;
tree[2].Sd=x;
tree[2].Sf=z;
tree[3].Sd=y;
tree[3].Sf=z;

tree[x].Mark = 1;
tree[y].Mark = 1;
tree[z].Mark = 1;
Xt[1].pl = x;
Xt[2].pl = y;
Xt[3].pl = z;


Ns = Nsmax;
tree[x].Pred = Ns;
tree[y].Pred = Ns;
tree[z].Pred = Ns;

tree[x].Long = (Xdst[x][y] + Xdst[x][z] - Xdst[z][y]) / 2.; 
tree[y].Long = (Xdst[x][y] + Xdst[y][z] - Xdst[z][x]) / 2.; 
tree[z].Long = (Xdst[x][z] + Xdst[y][z] - Xdst[x][y]) / 2.; 

NbPl = 3;
Na = 3;

/* initialisation */
for(ii=1; ii<=NbYt; ii++) {
	som = Xt[ii].idx;
	if(tree[som].Mark == 1) continue;
	lmax = 2. * Dmax + 1;
	for(aa = 1; aa <= Na; aa++) {
	   Sd = tree[aa].Sd;
	   Sf = tree[aa].Sf;
	   lkk = Xdst[som][Sd] + Xdst[som][Sf] - Xdst[Sd][Sf];
	   if((lmax - lkk) > EPSILON) {
		lmax = lkk;
		ar = aa;
		}
	   }
	Xt[som].bestar = ar;
	Xt[som].bestd = lmax;
	}

while(NbPl < NbYt) {

  lmax = 2. * Dmax + 1;
  for(ii=1; ii <= NbYt; ii++) {
	som = Xt[ii].idx;
	if(tree[som].Mark == 1) continue;
	if((lmax - Xt[som].bestd) > EPSILON) {
		cand = som;
		lmax = Xt[som].bestd;
		}
	}

  ar = Xt[cand].bestar;
  Sd = tree[ar].Sd;
  Sf = tree[ar].Sf;



  Na += 1;
  tree[Na].Sd = cand;
  tree[Na].Sf = tree[ar].Sd;

  Na += 1;
  tree[Na].Sd = cand;
  tree[Na].Sf = tree[ar].Sf;

  if(Growtree(cand, tree[ar].Sd, tree[ar].Sf) == 0) {
	FreeTab();
	return(0);
	}

  UpDist(cand);
  NbPl++;
  Xt[NbPl].pl = cand;
  tree[cand].Mark = 1;



  /* updating of best distances */
  for(ii=1; ii <= NbYt; ii++) {
	som = Xt[ii].idx;
	if(tree[som].Mark == 1) continue;

	lkk = Xdst[som][Sd] + Xdst[som][cand] - Xdst[Sd][cand];
	if((Xt[som].bestd - lkk) > EPSILON) {
		Xt[som].bestd = lkk;
		Xt[som].bestar = Na-1;
		}
	lkk = Xdst[som][Sf] + Xdst[som][cand] - Xdst[Sf][cand];
	if((Xt[som].bestd - lkk) > EPSILON) {
		Xt[som].bestd = lkk;
		Xt[som].bestar = Na;
		}
	}
  }


/* placing remaining sequences */


for(kk = NbYt+1; kk <= NbXt; kk++) {
   lmax = 2. * Dmax + 1;
   cand = Xt[kk].idx;

   if(Xt[kk].idx < 0) break;
   for(ii=1; ii <= NbPl -1; ii++) {
	somi = Xt[ii].pl;
	if(Xdst[somi][cand] < MEPSILON) continue;
	for(jj=ii+1; jj <= NbPl; jj++) {
	   somj = Xt[jj].pl;
	   if(Xdst[somj][cand]< MEPSILON) continue;
	   lkk = Xdst[somj][cand] + Xdst[somi][cand] - Xdst[somj][somi];
	   if((lmax - lkk) > EPSILON) {
		Sd = somi;
		Sf = somj;
		lmax = lkk;
		}
	   }
	}
   if(Growtree(cand, Sd, Sf) == 0) {
	FreeTab();
	return(0);
	}

   UpDist(cand);
   NbPl++;
   Xt[NbPl].pl = cand;
   }


CompTree();


FormatTree(treepar);

FreeTab();

return(1);
}
/********************** end TriangleTree ****************************/









/* loads the distance matrice, checks symetry, replace negative values by
   -1, initiate global variables */


int loadXt(double **dist, char **nom, int nbdist)
{
int ii, jj;

NbXt = nbdist;
Nsmax = 2 * NbXt - 2;
Nbm = Nb0 = 0;

Xt = (struct TABDIST *) check_alloc(NbXt+1, sizeof(struct TABDIST));
Xdst = (double **) check_alloc(NbXt+1, sizeof(double *));
Xmn = (char **) check_alloc(NbXt+1, sizeof(char *));


tree = (struct TREE *) check_alloc(Nsmax+1, sizeof(struct TREE));



for(ii=1; ii <= Nsmax; ii++) {
	tree[ii].Mark = 0;
	tree[ii].Pred = 0;
	tree[ii].Long = 0.;
	}

Dmax = -1.;

for(ii=1; ii <= NbXt; ii++) {
	Xdst[ii] = (double *) check_alloc(NbXt+1, sizeof(double));
	Xmn[ii] = (char *) check_alloc(MAXLNAME, sizeof(char));
	Xt[ii].inc = 0;
	Xt[ii].idx = ii;
	strcpy(Xmn[ii], nom[ii-1]);
	}

for(ii=1; ii <= NbXt - 1; ii++) {
	for(jj=ii+1;jj<=NbXt;jj++) {
		if(dist[ii-1][jj-1] != dist[jj-1][ii-1]) {
			fprintf(stderr, 
				"EXIT: Error, the distance matrix is not symetrical!\n");
			return(0);
			}
		Xdst[ii][jj] =  Xdst[jj][ii] =  dist[ii-1][jj-1];
		if(Xdst[ii][jj] < 0.) {

			Xdst[ii][jj] =  Xdst[jj][ii] = -1.;
			Xt[ii].inc += 1;
			Xt[jj].inc += 1;
			Nbm++;
			}


		if((Xdst[ii][jj] - Dmax) > EPSILON) Dmax = Xdst[ii][jj];
		}
	}

return(1);
}
/********************** end loadXt ****************************/


/* free Xt Xdst Xmn tree */
void FreeTab(void)
{
int ii;

for(ii=1; ii <= NbXt; ii++) {
	free((char *)Xdst[ii]);
	free((char *)Xmn[ii]);
	}


free((char *)Xdst);
free((char *)Xmn);
free((char *)Xt);
free((char *)tree);

}
/********************** end FreeTab ****************************/



/****************** sort data *********/
void sortXt(void)
{
int ii, jj, kk, ll, max, imax, qq, somi, somj;
int NN;
int flag;


/* sort by increasing number of known distances */

flag = 1;
while(flag == 1) {
	flag = 0;
	for(ii=1; ii <= NbXt -1; ii++) {
		if(Xt[ii].inc <= Xt[ii+1].inc) continue;
		SwapDist(ii, ii+1);
		flag = 1;
		}

	}

/* compute the number of sequences with complete distances */
for(ii=1; ii <= NbXt; ii++) {
	if(Xt[ii].inc != 0) break;
	}
Nbc = ii-1;

if(Nbc == NbXt) {
	NbYt = NbXt;
	return;
	}

if(Nbc == (NbXt - 2)) {
	NbYt = NbXt - 1;
	return;
	}


for(ii=Nbc+1; ii <= NbXt; ii++) {
	if(Xt[ii].idx < 0) continue;
	kk = Xt[ii].idx;
	for(jj= ii+1; jj <= NbXt; jj++) {
		if(Xt[jj].idx < 0) continue;
		ll = Xt[jj].idx;
		if(Xdst[kk][ll] == -1.) Xt[jj].idx = -1 * Xt[jj].idx;
		}
	}

kk = Nbc;
qq = NbXt + 1;

for(ii=Nbc + 1; ii <= NbXt; ii++) {
   if(Xt[ii].idx > 0) {
	kk++;
	Xt[kk].inc = Xt[ii].idx;
	}
   }

for(ii=NbXt; ii > Nbc; ii--) {
   if(Xt[ii].idx < 0) {
	qq--;
	Xt[qq].inc = Xt[ii].idx;
	}
   }


for(ii=Nbc+1; ii <= NbXt; ii++) {
	Xt[ii].idx = Xt[ii].inc;
	}

NbYt = kk;




for(ii=NbYt+1; ii <= NbXt; ii++) {
	Xt[ii].inc = 0;
	}
for(ii=1; ii<= NbYt; ii++) {
	somi = Xt[ii].idx;
	for(jj=NbYt+1; jj <= NbXt; jj++) {
		somj = -1 * Xt[jj].idx;
		if(Xdst[somi][somj] >= 0.) Xt[jj].inc += 1;
		}
	}


NN = NbYt + 1;
while(NN < NbXt) {
	max = 0;
	for(ii=NN; ii <= NbXt; ii++) {
		if(Xt[ii].inc > max) {
			max = Xt[ii].inc;
			imax = ii;
			}
		}
	if(max < 2) break;
	Xt[imax].idx = -1 * Xt[imax].idx;
	SwapDist(NN, imax);
	NN++;

	for(ii=NN; ii <= NbXt; ii++) {
		somi = -1 * Xt[ii].idx;
		somj = Xt[NN-1].idx;
		if(Xdst[somi][somj] >= 0.) Xt[ii].inc += 1;
		}

	}
if(Xt[NbXt].inc >= 2) {
	Xt[NbXt].idx = -1 * Xt[NbXt].idx;
	}

for(ii=1; ii<= NbXt; ii++) {
	if(Xt[ii].idx < 0) {
		jj = -1 * Xt[ii].idx;
		fprintf(stderr, 
			"WARNING: %d %s cannot be placed in the tree! ", 
			jj, Xmn[jj]);
		}
	}
}
/********************** end sortXt ****************************/










/* Select in Y a triplet that forms a triangle, with positive edges
lengths, metric and with minimal perimeter. */
 
int SelectTriple(int *x, int *y, int *z)
{
int ii, jj, kk;
int xx, yy, zz;
double dxy, dxz, dyz;
double permin, per;


permin = -1.;

if(NbYt < 3) {
	fprintf(stdout, "WARNING: less than 3 sequences in Yt ");
	return(0);
	}


for(ii = 1; ii <= NbYt - 2; ii++) {
   xx = Xt[ii].idx;

   for(jj = ii + 1; jj <= NbYt - 1; jj++) {
	yy = Xt[jj].idx;
	dxy = Xdst[xx][yy];
	if(dxy == 0.) continue;

	for(kk = jj + 1; kk <= NbYt; kk++) {
		zz = Xt[kk].idx;
		dxz = Xdst[xx][zz];
		dyz = Xdst[yy][zz];

		if(dxz == 0.) continue;
		if(dyz == 0.) continue;

		/* test      D(x,y) <= D(x,z) + D(y,z) */
		if((dxy - (dxz + dyz)) > EPSILON) continue;

		/* test     | D(x,z) - D(y,z) | <= D(x,y) */
		if( (fabs(dxz - dyz) - dxy) > EPSILON) continue;

		per = dxy + dxz + dyz;
		if(permin == -1. || (permin - per) > EPSILON) {
			permin = per;
			*x = Xt[ii].idx;
			*y = Xt[jj].idx;
			*z = Xt[kk].idx;
			}


		}
	}
   }

/* if no metric triangle, select the first triplet */
if(permin == -1.) {
	/*fprintf(stdout, "WARNING: no metric triangle ");*/
	*x = Xt[1].idx;
	*y = Xt[2].idx;
	*z = Xt[3].idx;
	}

return(1);
}
/********************** end SelectTriple ****************************/


int Growtree(int s, int t, int u)
{
double lgt, lgu, lg;
int kk, som, ssom, test;

Ns -= 1;

tree[s].Pred = Ns;
tree[s].Long = (Xdst[s][t] + Xdst[s][u] - Xdst[t][u]) / 2.;

if(tree[s].Long < 0.) tree[s].Long = 0.;

lgt = (Xdst[s][t] + Xdst[t][u] - Xdst[s][u]) / 2.;
lgu = (Xdst[u][t] + Xdst[s][u] - Xdst[t][s]) / 2.;


if(lgt < 0.) {
	lgu = Xdst[u][t];
	lgt = 0.;
	}

if(lgu < 0.) {
	lgt = Xdst[u][t];
	lgu = 0.;
	}


Path(t, u);


lg = lgt;
som = t;
test = 1;
while((lg - tree[som].Long) > EPSILON) {
	lg = lg - tree[som].Long;
	som = tree[som].Pred;
	if(tree[som].Ch != 1) {
		test = 0;
		break;
		}
	}


if(test == 0) {

   lg = lgu;
   som = u;
   test = 1;
   while((lg - tree[som].Long) > EPSILON) {
	lg = lg - tree[som].Long;
	som = tree[som].Pred;
	if(tree[som].Ch != 1) {
		test = 0;
		break;
		}

	}
   }


if(test == 0) {
	fprintf(stderr, "ERROR 100 !! \n");
	return(0);
	}


tree[Ns].Pred = tree[som].Pred;
tree[Ns].Long = tree[som].Long - lg;
tree[som].Pred = Ns;
tree[som].Long = lg;

return(1);
}
/********************** end Growtree ****************************/



/* Update distances */

void UpDist(int cand)
{
int ii, kk, som;
double lg;

for(ii = 1; ii <= NbPl; ii++) {
	som = Xt[ii].pl;
	Path(cand, som);
	lg = 0.;
	for(kk=1; kk <= Nsmax; kk++) {
		if(tree[kk].Ch != 1) continue;
		lg = lg + tree[kk].Long;
		}
	Xdst[cand][som] = Xdst[som][cand] = lg;

	}

}
/********************** end UpDist ****************************/




void Path(int t, int u) 
{
int kk, som, ssom;


for(kk = 1; kk <= Nsmax; kk++) {
	tree[kk].Ch = 0;
	}

som = t;

tree[som].Ch = 1;

for(;;) {
   ssom = tree[som].Pred;

   if(ssom <= 0) break;

   tree[ssom].Ch = 1;
   som = ssom;
   }


som = u;

tree[som].Ch = 1;

for(;;) {
   ssom = tree[som].Pred;

   if(ssom <= 0) break;

   tree[ssom].Ch = tree[ssom].Ch + 1;
   som = ssom;
   }


}
/********************** end Path ****************************/


void SwapDist(int ii, int jj)
{
int inc;		
int idx;		


inc = Xt[ii].inc;
Xt[ii].inc = Xt[jj].inc;
Xt[jj].inc = inc;

idx = Xt[ii].idx;
Xt[ii].idx = Xt[jj].idx;
Xt[jj].idx = idx;




}
/********************** end SwapDist ****************************/

void CompTree(void)
{
int ii, Sf, Sd, Suc;

for(ii = 1; ii <= Nsmax; ii++) {
	tree[ii].Fre = tree[ii].Suc = 0;
	}


for(ii = 1; ii <= Nsmax - 1; ii++) {
	Sd = ii;
	Sf = tree[ii].Pred;

	if(Sf == 0) continue;

	if (tree[Sf].Suc == 0)  tree[Sf].Suc = Sd;
	else {
		Suc = tree[Sf].Suc;
		while(tree[Suc].Fre > 0) {
			Suc = tree[Suc].Fre;
			}
		tree[Suc].Fre = Sd;
		}
	}





}
/********************** end CompTree ****************************/

void FormatTree(char *string)
{
int som, Fre;

string[0]=0;

som = Nsmax;
for(; ; ) {
	if(tree[som].Suc > 0) {
		sprintf(string, "%s(", string);
		tree[som].Suc = -1 * tree[som].Suc;
		som = -1 * tree[som].Suc;
		}

	else if(tree[som].Fre != 0){
		Fre = tree[som].Fre;
		if(tree[som].Suc == 0) sprintf(string, "%s%s:%.5f,", 
			     string, Xmn[som], tree[som].Long);
		else sprintf(string, "%s:%.5f,",string, tree[som].Long);
		som = Fre;
		}
	else if(tree[som].Pred != 0){
		if(tree[som].Suc == 0) sprintf(string, "%s%s:%.5f)", 
					string, Xmn[som], tree[som].Long);
		else sprintf(string, "%s:%.5f)",string, tree[som].Long);

		som = tree[som].Pred;
		}

	else break;
	}

strcat(string,";");



}
/********************** end FormatTree ****************************/


char *check_alloc(int nbrelt, int sizelt)
{
char *retval;
if( (retval=calloc(nbrelt,sizelt)) != NULL ) return(retval);
fprintf(stderr,"\nERROR: not enough memory (check_alloc).\n");
exit(1);
}
/********************** end check_alloc ****************************/



/******************** Example of function call ***********************/

main()
{  
  int test;

  double** d;
  char ctree[1000], *nom[5];
  int i, j;

  for(i=0;i<5;i++){
   nom[i]=check_alloc(MAXLNAME, sizeof(char));
   sprintf(nom[i], "species%d", i+1);
  }

  d=(double **) check_alloc(6, sizeof(double*));
  for(i=0;i<6;i++)
    d[i]=(double *) check_alloc(6, sizeof(double));

/* test distance matrix, unknown distances are indicated by 
   negative values */

  d[0][1]=d[1][0]=0.1;
  d[0][2]=d[2][0]=0.12;
  d[0][3]=d[3][0]=0.09;
  d[0][4]=d[4][0]=-1.0;
  d[1][2]=d[2][1]=0.22;
  d[1][3]=d[3][1]=0.06;
  d[1][4]=d[4][1]=0.13;
  d[2][3]=d[3][2]=0.11;
  d[2][4]=d[4][2]=0.1;
  d[3][4]=d[4][3]=0.07;

test = TriangleTree(d, nom, 5, ctree);


if(test == 1) printf("%s\n", ctree);

else printf("Warning: impossible to compute phylogenetic tree !\n");

exit(0);
} 
/********************** end main ****************************/
